@@ -2,18 +2,16 @@
  * \file Ellipsoid.hpp
  * \brief Header for GeographicLib::Ellipsoid class
  *
- * Copyright (c) Charles Karney (2012) <charles@karney.com> and licensed under
- * the MIT/X11 License.  For more information, see
- * http://geographiclib.sourceforge.net/
+ * Copyright (c) Charles Karney (2012-2023) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
+ * https://geographiclib.sourceforge.io/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_ELLIPSOID_HPP)
 #define GEOGRAPHICLIB_ELLIPSOID_HPP 1
 
 #include <GeographicLib/Constants.hpp>
-#include <GeographicLib/TransverseMercator.hpp>
-#include <GeographicLib/EllipticFunction.hpp>
-#include <GeographicLib/AlbersEqualArea.hpp>
+#include <GeographicLib/AuxLatitude.hpp>
 
 namespace GeographicLib {
 
@@ -21,16 +19,10 @@ namespace GeographicLib {
    * \brief Properties of an ellipsoid
    *
    * This class returns various properties of the ellipsoid and converts
-   * between various types of latitudes.  The latitude conversions are also
-   * possible using the various projections supported by %GeographicLib; but
-   * Ellipsoid provides more direct access (sometimes using private functions
-   * of the projection classes).  Ellipsoid::RectifyingLatitude,
-   * Ellipsoid::InverseRectifyingLatitude, and Ellipsoid::MeridianDistance
-   * provide functionality which can be provided by the Geodesic class.
-   * However Geodesic uses a series approximation (valid for abs \e f < 1/150),
-   * whereas Ellipsoid computes these quantities using EllipticFunction which
-   * provides accurate results even when \e f is large.  Use of this class
-   * should be limited to &minus;3 < \e f < 3/4 (i.e., 1/4 < b/a < 4).
+   * between various types of latitudes.  This is for the most part a thin
+   * wrapper on top of the AuxLatitude class which is called with \e exact =
+   * true so that the results are valid for arbitrary flattenings &minus;100 <
+   * \e f < 99/100 (i.e., 1/100 < b/a < 100).
    *
    * Example of use:
    * \include example-Ellipsoid.cpp
@@ -40,20 +32,10 @@ namespace GeographicLib {
   private:
     typedef Math::real real;
     static const int numit_ = 10;
-    static const real stol_;
-    real _a, _f, _f1, _f12, _e2, _e12, _n, _b;
-    TransverseMercator _tm;
-    EllipticFunction _ell;
-    AlbersEqualArea _au;
-    static real tand(real x) throw() {
-      return
-        std::abs(x) == real(90) ? (x < 0 ?
-                                   - TransverseMercator::overflow_
-                                   : TransverseMercator::overflow_) :
-        std::tan(x * Math::degree<real>());
-    }
-    static real atand(real x) throw()
-    { return std::atan(x) / Math::degree<real>(); }
+    real stol_;
+    real _a, _f,_b, _e2, _e12, _n;
+    AuxLatitude _aux;
+    real _rm, _c2;
 
   public:
     /** \name Constructor
@@ -61,13 +43,12 @@ namespace GeographicLib {
     ///@{
 
     /**
-     * Constructor for a ellipsoid with
+     * Constructor for an ellipsoid with
      *
      * @param[in] a equatorial radius (meters).
      * @param[in] f flattening of ellipsoid.  Setting \e f = 0 gives a sphere.
-     *   Negative \e f gives a prolate ellipsoid.  If \e f > 1, set flattening
-     *   to 1/\e f.
-     * @exception GeographicErr if \e a or (1 &minus; \e f ) \e a is not
+     *   Negative \e f gives a prolate ellipsoid.
+     * @exception GeographicErr if \e a or (1 &minus; \e f) \e a is not
      *   positive.
      **********************************************************************/
     Ellipsoid(real a, real f);
@@ -81,34 +62,34 @@ namespace GeographicLib {
      * @return \e a the equatorial radius of the ellipsoid (meters).  This is
      *   the value used in the constructor.
      **********************************************************************/
-    Math::real MajorRadius() const throw() { return _a; }
+    Math::real EquatorialRadius() const { return _a; }
 
     /**
      * @return \e b the polar semi-axis (meters).
      **********************************************************************/
-    Math::real MinorRadius() const throw() { return _b; }
+    Math::real PolarRadius() const { return _b; }
 
     /**
      * @return \e L the distance between the equator and a pole along a
      *   meridian (meters).  For a sphere \e L = (&pi;/2) \e a.  The radius
      *   of a sphere with the same meridian length is \e L / (&pi;/2).
      **********************************************************************/
-    Math::real QuarterMeridian() const throw();
+    Math::real QuarterMeridian() const;
 
     /**
      * @return \e A the total area of the ellipsoid (meters<sup>2</sup>).  For
      *   a sphere \e A = 4&pi; <i>a</i><sup>2</sup>.  The radius of a sphere
      *   with the same area is sqrt(\e A / (4&pi;)).
      **********************************************************************/
-    Math::real Area() const throw();
+    Math::real Area() const;
 
     /**
      * @return \e V the total volume of the ellipsoid (meters<sup>3</sup>).
      *   For a sphere \e V = (4&pi; / 3) <i>a</i><sup>3</sup>.  The radius of
      *   a sphere with the same volume is cbrt(\e V / (4&pi;/3)).
      **********************************************************************/
-    Math::real Volume() const throw()
-    { return (4 * Math::pi<real>()) * Math::sq(_a) * _b / 3; }
+    Math::real Volume() const
+    { return (4 * Math::pi()) * Math::sq(_a) * _b / 3; }
     ///@}
 
     /** \name %Ellipsoid shape
@@ -121,21 +102,21 @@ namespace GeographicLib {
      *   positive, or negative for a sphere, oblate ellipsoid, or prolate
      *   ellipsoid.
      **********************************************************************/
-    Math::real Flattening() const throw() { return _f; }
+    Math::real Flattening() const { return _f; }
 
     /**
      * @return \e f ' = (\e a &minus; \e b) / \e b, the second flattening of
      *   the ellipsoid.  This is zero, positive, or negative for a sphere,
      *   oblate ellipsoid, or prolate ellipsoid.
      **********************************************************************/
-    Math::real SecondFlattening() const throw() { return _f / (1 - _f); }
+    Math::real SecondFlattening() const { return _f / (1 - _f); }
 
     /**
      * @return \e n = (\e a &minus; \e b) / (\e a + \e b), the third flattening
      *   of the ellipsoid.  This is zero, positive, or negative for a sphere,
      *   oblate ellipsoid, or prolate ellipsoid.
      **********************************************************************/
-    Math::real ThirdFlattening() const throw() { return _n; }
+    Math::real ThirdFlattening() const { return _n; }
 
     /**
      * @return <i>e</i><sup>2</sup> = (<i>a</i><sup>2</sup> &minus;
@@ -143,7 +124,7 @@ namespace GeographicLib {
      *   of the ellipsoid.  This is zero, positive, or negative for a sphere,
      *   oblate ellipsoid, or prolate ellipsoid.
      **********************************************************************/
-    Math::real EccentricitySq() const throw() { return _e2; }
+    Math::real EccentricitySq() const { return _e2; }
 
     /**
      * @return <i>e'</i> <sup>2</sup> = (<i>a</i><sup>2</sup> &minus;
@@ -151,7 +132,7 @@ namespace GeographicLib {
      *   squared of the ellipsoid.  This is zero, positive, or negative for a
      *   sphere, oblate ellipsoid, or prolate ellipsoid.
      **********************************************************************/
-    Math::real SecondEccentricitySq() const throw() { return _e12; }
+    Math::real SecondEccentricitySq() const { return _e12; }
 
     /**
      * @return <i>e''</i> <sup>2</sup> = (<i>a</i><sup>2</sup> &minus;
@@ -160,7 +141,7 @@ namespace GeographicLib {
      *   positive, or negative for a sphere, oblate ellipsoid, or prolate
      *   ellipsoid.
      **********************************************************************/
-    Math::real ThirdEccentricitySq() const throw() { return _e2 / (2 - _e2); }
+    Math::real ThirdEccentricitySq() const { return _e2 / (2 - _e2); }
     ///@}
 
     /** \name Latitude conversion.
@@ -171,7 +152,7 @@ namespace GeographicLib {
      * @param[in] phi the geographic latitude (degrees).
      * @return &beta; the parametric latitude (degrees).
      *
-     * The geographic latitude, &phi;, is the angle beween the equatorial
+     * The geographic latitude, &phi;, is the angle between the equatorial
      * plane and a vector normal to the surface of the ellipsoid.
      *
      * The parametric latitude (also called the reduced latitude), &beta;,
@@ -187,7 +168,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &beta; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real ParametricLatitude(real phi) const throw();
+    Math::real ParametricLatitude(real phi) const;
 
     /**
      * @param[in] beta the parametric latitude (degrees).
@@ -197,13 +178,13 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &phi; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real InverseParametricLatitude(real beta) const throw();
+    Math::real InverseParametricLatitude(real beta) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
      * @return &theta; the geocentric latitude (degrees).
      *
-     * The geocentric latitude, &theta;, is the angle beween the equatorial
+     * The geocentric latitude, &theta;, is the angle between the equatorial
      * plane and a line between the center of the ellipsoid and a point on the
      * ellipsoid.  For a sphere &theta; = &phi;.
      *
@@ -211,7 +192,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &theta; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real GeocentricLatitude(real phi) const throw();
+    Math::real GeocentricLatitude(real phi) const;
 
     /**
      * @param[in] theta the geocentric latitude (degrees).
@@ -221,7 +202,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &phi; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real InverseGeocentricLatitude(real theta) const throw();
+    Math::real InverseGeocentricLatitude(real theta) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -237,7 +218,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &mu; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real RectifyingLatitude(real phi) const throw();
+    Math::real RectifyingLatitude(real phi) const;
 
     /**
      * @param[in] mu the rectifying latitude (degrees).
@@ -247,7 +228,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &phi; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real InverseRectifyingLatitude(real mu) const throw();
+    Math::real InverseRectifyingLatitude(real mu) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -263,7 +244,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &xi; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real AuthalicLatitude(real phi) const throw();
+    Math::real AuthalicLatitude(real phi) const;
 
     /**
      * @param[in] xi the authalic latitude (degrees).
@@ -273,7 +254,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &phi; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real InverseAuthalicLatitude(real xi) const throw();
+    Math::real InverseAuthalicLatitude(real xi) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -288,7 +269,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &chi; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real ConformalLatitude(real phi) const throw();
+    Math::real ConformalLatitude(real phi) const;
 
     /**
      * @param[in] chi the conformal latitude (degrees).
@@ -298,7 +279,7 @@ namespace GeographicLib {
      * result is undefined if this condition does not hold.  The returned value
      * &phi; lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real InverseConformalLatitude(real chi) const throw();
+    Math::real InverseConformalLatitude(real chi) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -310,18 +291,21 @@ namespace GeographicLib {
      * defines the Mercator projection.  For a sphere &psi; =
      * sinh<sup>&minus;1</sup> tan &phi;.
      *
-     * &phi; must lie in the range [&minus;90&deg;, 90&deg;]; the
-     * result is undefined if this condition does not hold.
+     * &phi; must lie in the range [&minus;90&deg;, 90&deg;]; the result is
+     * undefined if this condition does not hold.  The value returned for &phi;
+     * = &plusmn;90&deg; is some (positive or negative) large but finite value,
+     * such that InverseIsometricLatitude returns the original value of &phi;.
      **********************************************************************/
-    Math::real IsometricLatitude(real phi) const throw();
+    Math::real IsometricLatitude(real phi) const;
 
     /**
      * @param[in] psi the isometric latitude (degrees).
      * @return &phi; the geographic latitude (degrees).
      *
-     * The returned value &phi; lies in [&minus;90&deg;, 90&deg;].
+     * The returned value &phi; lies in [&minus;90&deg;, 90&deg;].  For a
+     * sphere &phi; = tan<sup>&minus;1</sup> sinh &psi;.
      **********************************************************************/
-    Math::real InverseIsometricLatitude(real psi) const throw();
+    Math::real InverseIsometricLatitude(real psi) const;
     ///@}
 
     /** \name Other quantities.
@@ -337,7 +321,7 @@ namespace GeographicLib {
      * &phi; must lie in the range [&minus;90&deg;, 90&deg;]; the
      * result is undefined if this condition does not hold.
      **********************************************************************/
-    Math::real CircleRadius(real phi) const throw();
+    Math::real CircleRadius(real phi) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -348,7 +332,7 @@ namespace GeographicLib {
      * &phi; must lie in the range [&minus;90&deg;, 90&deg;]; the
      * result is undefined if this condition does not hold.
      **********************************************************************/
-    Math::real CircleHeight(real phi) const throw();
+    Math::real CircleHeight(real phi) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -360,7 +344,7 @@ namespace GeographicLib {
      * &phi; must lie in the range [&minus;90&deg;, 90&deg;]; the
      * result is undefined if this condition does not hold.
      **********************************************************************/
-    Math::real MeridianDistance(real phi) const throw();
+    Math::real MeridianDistance(real phi) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -373,7 +357,7 @@ namespace GeographicLib {
      * &phi; must lie in the range [&minus;90&deg;, 90&deg;]; the
      * result is undefined if this condition does not hold.
      **********************************************************************/
-    Math::real MeridionalCurvatureRadius(real phi) const throw();
+    Math::real MeridionalCurvatureRadius(real phi) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -386,7 +370,7 @@ namespace GeographicLib {
      * &phi; must lie in the range [&minus;90&deg;, 90&deg;]; the
      * result is undefined if this condition does not hold.
      **********************************************************************/
-    Math::real TransverseCurvatureRadius(real phi) const throw();
+    Math::real TransverseCurvatureRadius(real phi) const;
 
     /**
      * @param[in] phi the geographic latitude (degrees).
@@ -396,11 +380,10 @@ namespace GeographicLib {
      *   section at latitude &phi; inclined at an angle \e azi to the
      *   meridian (meters).
      *
-     * &phi; must lie in the range [&minus;90&deg;, 90&deg;] and \e
-     * azi must lie in the range [&minus;540&deg;, 540&deg;); the
-     * result is undefined if either of conditions does not hold.
+     * &phi; must lie in the range [&minus;90&deg;, 90&deg;]; the result is
+     * undefined this condition does not hold.
      **********************************************************************/
-    Math::real NormalCurvatureRadius(real phi, real azi) const throw();
+    Math::real NormalCurvatureRadius(real phi, real azi) const;
     ///@}
 
     /** \name Eccentricity conversions.
@@ -415,7 +398,7 @@ namespace GeographicLib {
      * \e f ' should lie in (&minus;1, &infin;).
      * The returned value \e f lies in (&minus;&infin;, 1).
      **********************************************************************/
-    static Math::real SecondFlatteningToFlattening(real fp) throw()
+    static Math::real SecondFlatteningToFlattening(real fp)
     { return fp / (1 + fp); }
 
     /**
@@ -425,7 +408,7 @@ namespace GeographicLib {
      * \e f should lie in (&minus;&infin;, 1).
      * The returned value \e f ' lies in (&minus;1, &infin;).
      **********************************************************************/
-    static Math::real FlatteningToSecondFlattening(real f) throw()
+    static Math::real FlatteningToSecondFlattening(real f)
     { return f / (1 - f); }
 
     /**
@@ -436,7 +419,7 @@ namespace GeographicLib {
      * \e n should lie in (&minus;1, 1).
      * The returned value \e f lies in (&minus;&infin;, 1).
      **********************************************************************/
-    static Math::real ThirdFlatteningToFlattening(real n) throw()
+    static Math::real ThirdFlatteningToFlattening(real n)
     { return 2 * n / (1 + n); }
 
     /**
@@ -447,7 +430,7 @@ namespace GeographicLib {
      * \e f should lie in (&minus;&infin;, 1).
      * The returned value \e n lies in (&minus;1, 1).
      **********************************************************************/
-    static Math::real FlatteningToThirdFlattening(real f) throw()
+    static Math::real FlatteningToThirdFlattening(real f)
     { return f / (2 - f); }
 
     /**
@@ -459,8 +442,8 @@ namespace GeographicLib {
      * <i>e</i><sup>2</sup> should lie in (&minus;&infin;, 1).
      * The returned value \e f lies in (&minus;&infin;, 1).
      **********************************************************************/
-    static Math::real EccentricitySqToFlattening(real e2) throw()
-    { return e2 / (std::sqrt(1 - e2) + 1); }
+    static Math::real EccentricitySqToFlattening(real e2)
+    { using std::sqrt; return e2 / (sqrt(1 - e2) + 1); }
 
     /**
      * @param[in] f = (\e a &minus; \e b) / \e a, the flattening.
@@ -471,7 +454,7 @@ namespace GeographicLib {
      * \e f should lie in (&minus;&infin;, 1).
      * The returned value <i>e</i><sup>2</sup> lies in (&minus;&infin;, 1).
      **********************************************************************/
-    static Math::real FlatteningToEccentricitySq(real f) throw()
+    static Math::real FlatteningToEccentricitySq(real f)
     { return f * (2 - f); }
 
     /**
@@ -483,8 +466,8 @@ namespace GeographicLib {
      * <i>e'</i> <sup>2</sup> should lie in (&minus;1, &infin;).
      * The returned value \e f lies in (&minus;&infin;, 1).
      **********************************************************************/
-    static Math::real SecondEccentricitySqToFlattening(real ep2) throw()
-    { return ep2 / (std::sqrt(1 + ep2) + 1 + ep2); }
+    static Math::real SecondEccentricitySqToFlattening(real ep2)
+    { using std::sqrt; return ep2 / (sqrt(1 + ep2) + 1 + ep2); }
 
     /**
      * @param[in] f = (\e a &minus; \e b) / \e a, the flattening.
@@ -495,7 +478,7 @@ namespace GeographicLib {
      * \e f should lie in (&minus;&infin;, 1).
      * The returned value <i>e'</i> <sup>2</sup> lies in (&minus;1, &infin;).
      **********************************************************************/
-    static Math::real FlatteningToSecondEccentricitySq(real f) throw()
+    static Math::real FlatteningToSecondEccentricitySq(real f)
     { return f * (2 - f) / Math::sq(1 - f); }
 
     /**
@@ -507,8 +490,10 @@ namespace GeographicLib {
      * <i>e''</i> <sup>2</sup> should lie in (&minus;1, 1).
      * The returned value \e f lies in (&minus;&infin;, 1).
      **********************************************************************/
-    static Math::real ThirdEccentricitySqToFlattening(real epp2) throw()
-    { return 2 * epp2 / (sqrt((1 - epp2) * (1 + epp2)) + 1 + epp2); }
+    static Math::real ThirdEccentricitySqToFlattening(real epp2) {
+      using std::sqrt;
+      return 2 * epp2 / (sqrt((1 - epp2) * (1 + epp2)) + 1 + epp2);
+    }
 
     /**
      * @param[in] f = (\e a &minus; \e b) / \e a, the flattening.
@@ -519,7 +504,7 @@ namespace GeographicLib {
      * \e f should lie in (&minus;&infin;, 1).
      * The returned value <i>e''</i> <sup>2</sup> lies in (&minus;1, 1).
      **********************************************************************/
-    static Math::real FlatteningToThirdEccentricitySq(real f) throw()
+    static Math::real FlatteningToThirdEccentricitySq(real f)
     { return f * (2 - f) / (1 + Math::sq(1 - f)); }
 
     ///@}
@@ -528,8 +513,7 @@ namespace GeographicLib {
      * A global instantiation of Ellipsoid with the parameters for the WGS84
      * ellipsoid.
      **********************************************************************/
-    static const Ellipsoid WGS84;
-
+    static const Ellipsoid& WGS84();
   };
 
 } // namespace GeographicLib
