@@ -2,9 +2,9 @@
  * \file GeodesicLineExact.hpp
  * \brief Header for GeographicLib::GeodesicLineExact class
  *
- * Copyright (c) Charles Karney (2012) <charles@karney.com> and licensed under
- * the MIT/X11 License.  For more information, see
- * http://geographiclib.sourceforge.net/
+ * Copyright (c) Charles Karney (2012-2023) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
+ * https://geographiclib.sourceforge.io/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_GEODESICLINEEXACT_HPP)
@@ -13,6 +13,7 @@
 #include <GeographicLib/Constants.hpp>
 #include <GeographicLib/GeodesicExact.hpp>
 #include <GeographicLib/EllipticFunction.hpp>
+#include <vector>
 
 namespace GeographicLib {
 
@@ -32,21 +33,33 @@ namespace GeographicLib {
    * GeodesicLineExact (via the -E option).
    **********************************************************************/
 
-  class GEOGRAPHICLIB_EXPORT GeodesicLineExact {
+  class GeodesicLineExact {
   private:
     typedef Math::real real;
     friend class GeodesicExact;
-    static const int nC4_ = GeodesicExact::nC4_;
+    int _nC4;
 
+    real tiny_;
     real _lat1, _lon1, _azi1;
     real _a, _f, _b, _c2, _f1, _e2, _salp0, _calp0, _k2,
       _salp1, _calp1, _ssig1, _csig1, _dn1, _stau1, _ctau1,
       _somg1, _comg1, _cchi1,
-      _A4, _B41, _E0, _D0, _H0, _E1, _D1, _H1;
-    real _C4a[nC4_];            // all the elements of _C4a are used
-    EllipticFunction _E;
+      _aA4, _eE0, _dD0, _hH0, _eE1, _dD1, _hH1;
+    real _a13, _s13;
+    real _bB41;
+    std::vector<real> _cC4a;
+    EllipticFunction _eE;
     unsigned _caps;
 
+    void LineInit(const GeodesicExact& g,
+                  real lat1, real lon1,
+                  real azi1, real salp1, real calp1,
+                  unsigned caps);
+    GeodesicLineExact(const GeodesicExact& g,
+                      real lat1, real lon1,
+                      real azi1, real salp1, real calp1,
+                      unsigned caps, bool arcmode, real s13_a13);
+
     enum captype {
       CAP_NONE = GeodesicExact::CAP_NONE,
       CAP_E    = GeodesicExact::CAP_E,
@@ -54,7 +67,9 @@ namespace GeographicLib {
       CAP_H    = GeodesicExact::CAP_H,
       CAP_C4   = GeodesicExact::CAP_C4,
       CAP_ALL  = GeodesicExact::CAP_ALL,
+      CAP_MASK = GeodesicExact::CAP_MASK,
       OUT_ALL  = GeodesicExact::OUT_ALL,
+      OUT_MASK = GeodesicExact::OUT_MASK,
     };
   public:
 
@@ -94,6 +109,13 @@ namespace GeographicLib {
        * @hideinitializer
        **********************************************************************/
       DISTANCE      = GeodesicExact::DISTANCE,
+      /**
+       * A combination of the common capabilities: GeodesicLineExact::LATITUDE,
+       * GeodesicLineExact::LONGITUDE, GeodesicLineExact::AZIMUTH,
+       * GeodesicLineExact::DISTANCE.
+       * @hideinitializer
+       **********************************************************************/
+      STANDARD      = GeodesicExact::STANDARD,
       /**
        * Allow distance \e s12 to be used as input in the direct geodesic
        * problem.
@@ -116,7 +138,13 @@ namespace GeographicLib {
        **********************************************************************/
       AREA          = GeodesicExact::AREA,
       /**
-       * All capabilities, calculate everything.
+       * Unroll \e lon2 in the direct calculation.
+       * @hideinitializer
+       **********************************************************************/
+      LONG_UNROLL = GeodesicExact::LONG_UNROLL,
+      /**
+       * All capabilities, calculate everything.  (LONG_UNROLL is not
+       * included in this mask.)
        * @hideinitializer
        **********************************************************************/
       ALL           = GeodesicExact::ALL,
@@ -140,18 +168,17 @@ namespace GeographicLib {
      *   possess, i.e., which quantities can be returned in calls to
      *   GeodesicLine::Position.
      *
-     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-     * azi1 should be in the range [&minus;540&deg;, 540&deg;).
+     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].
      *
      * The GeodesicLineExact::mask values are
      * - \e caps |= GeodesicLineExact::LATITUDE for the latitude \e lat2; this
      *   is added automatically;
      * - \e caps |= GeodesicLineExact::LONGITUDE for the latitude \e lon2;
-     * - \e caps |= GeodesicLineExact::AZIMUTH for the latitude \e azi2; this is
-     *   added automatically;
+     * - \e caps |= GeodesicLineExact::AZIMUTH for the latitude \e azi2; this
+     *   is added automatically;
      * - \e caps |= GeodesicLineExact::DISTANCE for the distance \e s12;
      * - \e caps |= GeodesicLineExact::REDUCEDLENGTH for the reduced length \e
-         m12;
+     *   m12;
      * - \e caps |= GeodesicLineExact::GEODESICSCALE for the geodesic scales \e
      *   M12 and \e M21;
      * - \e caps |= GeodesicLineExact::AREA for the area \e S12;
@@ -166,9 +193,9 @@ namespace GeographicLib {
      * fixed, writing \e lat1 = &plusmn;(90&deg; &minus; &epsilon;), and taking
      * the limit &epsilon; &rarr; 0+.
      **********************************************************************/
+    GEOGRAPHICLIB_EXPORT
     GeodesicLineExact(const GeodesicExact& g, real lat1, real lon1, real azi1,
-                      unsigned caps = ALL)
-      throw();
+                      unsigned caps = ALL);
 
     /**
      * A default constructor.  If GeodesicLineExact::Position is called on the
@@ -177,7 +204,7 @@ namespace GeographicLib {
      * GeodesicExact::Line.  Use Init() to test whether object is still in this
      * uninitialized state.
      **********************************************************************/
-    GeodesicLineExact() throw() : _caps(0U) {}
+    GEOGRAPHICLIB_EXPORT GeodesicLineExact() : _caps(0U) {}
     ///@}
 
     /** \name Position in terms of distance
@@ -188,7 +215,7 @@ namespace GeographicLib {
      * Compute the position of point 2 which is a distance \e s12 (meters)
      * from point 1.
      *
-     * @param[in] s12 distance between point 1 and point 2 (meters); it can be
+     * @param[in] s12 distance from point 1 to point 2 (meters); it can be
      *   signed.
      * @param[out] lat2 latitude of point 2 (degrees).
      * @param[out] lon2 longitude of point 2 (degrees); requires that the
@@ -207,10 +234,10 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>); requires
      *   that the GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::AREA.
-     * @return \e a12 arc length of between point 1 and point 2 (degrees).
+     * @return \e a12 arc length from point 1 to point 2 (degrees).
      *
      * The values of \e lon2 and \e azi2 returned are in the range
-     * [&minus;180&deg;, 180&deg;).
+     * [&minus;180&deg;, 180&deg;].
      *
      * The GeodesicLineExact object \e must have been constructed with \e caps
      * |= GeodesicLineExact::DISTANCE_IN; otherwise Math::NaN() is returned and
@@ -223,10 +250,10 @@ namespace GeographicLib {
      * Note, however, that the arc length is always computed and returned as
      * the function value.
      **********************************************************************/
-    Math::real Position(real s12,
-                        real& lat2, real& lon2, real& azi2,
-                        real& m12, real& M12, real& M21,
-                        real& S12) const throw() {
+    Math::real GEOGRAPHICLIB_EXPORT Position(real s12, real& lat2, real& lon2,
+                                             real& azi2,
+                                             real& m12, real& M12, real& M21,
+                                             real& S12) const {
       real t;
       return GenPosition(false, s12,
                          LATITUDE | LONGITUDE | AZIMUTH |
@@ -237,7 +264,8 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::Position.
      **********************************************************************/
-    Math::real Position(real s12, real& lat2, real& lon2) const throw() {
+    Math::real GEOGRAPHICLIB_EXPORT Position(real s12, real& lat2, real& lon2)
+      const {
       real t;
       return GenPosition(false, s12,
                          LATITUDE | LONGITUDE,
@@ -247,8 +275,8 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::Position.
      **********************************************************************/
-    Math::real Position(real s12, real& lat2, real& lon2,
-                        real& azi2) const throw() {
+    Math::real GEOGRAPHICLIB_EXPORT Position(real s12, real& lat2, real& lon2,
+                                             real& azi2) const {
       real t;
       return GenPosition(false, s12,
                          LATITUDE | LONGITUDE | AZIMUTH,
@@ -258,8 +286,8 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::Position.
      **********************************************************************/
-    Math::real Position(real s12, real& lat2, real& lon2,
-                        real& azi2, real& m12) const throw() {
+    Math::real GEOGRAPHICLIB_EXPORT Position(real s12, real& lat2, real& lon2,
+                                             real& azi2, real& m12) const {
       real t;
       return GenPosition(false, s12,
                          LATITUDE | LONGITUDE |
@@ -270,9 +298,9 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::Position.
      **********************************************************************/
-    Math::real Position(real s12, real& lat2, real& lon2,
-                        real& azi2, real& M12, real& M21)
-      const throw() {
+    Math::real GEOGRAPHICLIB_EXPORT Position(real s12, real& lat2, real& lon2,
+                                             real& azi2, real& M12, real& M21)
+      const {
       real t;
       return GenPosition(false, s12,
                          LATITUDE | LONGITUDE |
@@ -283,17 +311,16 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::Position.
      **********************************************************************/
-    Math::real Position(real s12,
-                        real& lat2, real& lon2, real& azi2,
-                        real& m12, real& M12, real& M21)
-      const throw() {
+    Math::real GEOGRAPHICLIB_EXPORT Position(real s12,
+                                             real& lat2, real& lon2, real& azi2,
+                                             real& m12, real& M12, real& M21)
+      const {
       real t;
       return GenPosition(false, s12,
                          LATITUDE | LONGITUDE | AZIMUTH |
                          REDUCEDLENGTH | GEODESICSCALE,
                          lat2, lon2, azi2, t, m12, M12, M21, t);
     }
-
     ///@}
 
     /** \name Position in terms of arc length
@@ -304,14 +331,14 @@ namespace GeographicLib {
      * Compute the position of point 2 which is an arc length \e a12 (degrees)
      * from point 1.
      *
-     * @param[in] a12 arc length between point 1 and point 2 (degrees); it can
+     * @param[in] a12 arc length from point 1 to point 2 (degrees); it can
      *   be signed.
      * @param[out] lat2 latitude of point 2 (degrees).
      * @param[out] lon2 longitude of point 2 (degrees); requires that the
      *   GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::LONGITUDE.
      * @param[out] azi2 (forward) azimuth at point 2 (degrees).
-     * @param[out] s12 distance between point 1 and point 2 (meters); requires
+     * @param[out] s12 distance from point 1 to point 2 (meters); requires
      *   that the GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::DISTANCE.
      * @param[out] m12 reduced length of geodesic (meters); requires that the
@@ -328,7 +355,7 @@ namespace GeographicLib {
      *   GeodesicLineExact::AREA.
      *
      * The values of \e lon2 and \e azi2 returned are in the range
-     * [&minus;180&deg;, 180&deg;).
+     * [&minus;180&deg;, 180&deg;].
      *
      * Requesting a value which the GeodesicLineExact object is not capable of
      * computing is not an error; the corresponding argument will not be
@@ -337,9 +364,10 @@ namespace GeographicLib {
      * The following functions are overloaded versions of
      * GeodesicLineExact::ArcPosition which omit some of the output parameters.
      **********************************************************************/
-    void ArcPosition(real a12, real& lat2, real& lon2, real& azi2,
-                     real& s12, real& m12, real& M12, real& M21,
-                     real& S12) const throw() {
+    void GEOGRAPHICLIB_EXPORT
+    ArcPosition(real a12, real& lat2, real& lon2, real& azi2,
+                real& s12, real& m12, real& M12, real& M21,
+                real& S12) const {
       GenPosition(true, a12,
                   LATITUDE | LONGITUDE | AZIMUTH | DISTANCE |
                   REDUCEDLENGTH | GEODESICSCALE | AREA,
@@ -349,8 +377,8 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::ArcPosition.
      **********************************************************************/
-    void ArcPosition(real a12, real& lat2, real& lon2)
-      const throw() {
+    void GEOGRAPHICLIB_EXPORT ArcPosition(real a12, real& lat2, real& lon2)
+      const {
       real t;
       GenPosition(true, a12,
                   LATITUDE | LONGITUDE,
@@ -360,9 +388,9 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::ArcPosition.
      **********************************************************************/
-    void ArcPosition(real a12,
-                     real& lat2, real& lon2, real& azi2)
-      const throw() {
+    void GEOGRAPHICLIB_EXPORT ArcPosition(real a12,
+                                          real& lat2, real& lon2, real& azi2)
+      const {
       real t;
       GenPosition(true, a12,
                   LATITUDE | LONGITUDE | AZIMUTH,
@@ -372,8 +400,9 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::ArcPosition.
      **********************************************************************/
-    void ArcPosition(real a12, real& lat2, real& lon2, real& azi2,
-                     real& s12) const throw() {
+    void GEOGRAPHICLIB_EXPORT ArcPosition(real a12,
+                                          real& lat2, real& lon2, real& azi2,
+                                          real& s12) const {
       real t;
       GenPosition(true, a12,
                   LATITUDE | LONGITUDE | AZIMUTH | DISTANCE,
@@ -383,8 +412,9 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::ArcPosition.
      **********************************************************************/
-    void ArcPosition(real a12, real& lat2, real& lon2, real& azi2,
-                     real& s12, real& m12) const throw() {
+    void GEOGRAPHICLIB_EXPORT ArcPosition(real a12,
+                                          real& lat2, real& lon2, real& azi2,
+                                          real& s12, real& m12) const {
       real t;
       GenPosition(true, a12,
                   LATITUDE | LONGITUDE | AZIMUTH |
@@ -395,9 +425,10 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::ArcPosition.
      **********************************************************************/
-    void ArcPosition(real a12, real& lat2, real& lon2, real& azi2,
-                     real& s12, real& M12, real& M21)
-      const throw() {
+    void GEOGRAPHICLIB_EXPORT ArcPosition(real a12,
+                                          real& lat2, real& lon2, real& azi2,
+                                          real& s12, real& M12, real& M21)
+      const {
       real t;
       GenPosition(true, a12,
                   LATITUDE | LONGITUDE | AZIMUTH |
@@ -408,9 +439,10 @@ namespace GeographicLib {
     /**
      * See the documentation for GeodesicLineExact::ArcPosition.
      **********************************************************************/
-    void ArcPosition(real a12, real& lat2, real& lon2, real& azi2,
-                     real& s12, real& m12, real& M12, real& M21)
-      const throw() {
+    void GEOGRAPHICLIB_EXPORT
+    ArcPosition(real a12, real& lat2, real& lon2, real& azi2,
+                real& s12, real& m12, real& M12, real& M21)
+      const {
       real t;
       GenPosition(true, a12,
                   LATITUDE | LONGITUDE | AZIMUTH |
@@ -428,8 +460,9 @@ namespace GeographicLib {
      * GeodesicLineExact::ArcPosition are defined in terms of this function.
      *
      * @param[in] arcmode boolean flag determining the meaning of the second
-     *   parameter; if arcmode is false, then the GeodesicLineExact object must
-     *   have been constructed with \e caps |= GeodesicLineExact::DISTANCE_IN.
+     *   parameter; if \e arcmode is false, then the GeodesicLineExact object
+     *   must have been constructed with \e caps |=
+     *   GeodesicLineExact::DISTANCE_IN.
      * @param[in] s12_a12 if \e arcmode is false, this is the distance between
      *   point 1 and point 2 (meters); otherwise it is the arc length between
      *   point 1 and point 2 (degrees); it can be signed.
@@ -440,7 +473,7 @@ namespace GeographicLib {
      *   GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::LONGITUDE.
      * @param[out] azi2 (forward) azimuth at point 2 (degrees).
-     * @param[out] s12 distance between point 1 and point 2 (meters); requires
+     * @param[out] s12 distance from point 1 to point 2 (meters); requires
      *   that the GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::DISTANCE.
      * @param[out] m12 reduced length of geodesic (meters); requires that the
@@ -455,7 +488,7 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>); requires
      *   that the GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::AREA.
-     * @return \e a12 arc length of between point 1 and point 2 (degrees).
+     * @return \e a12 arc length from point 1 to point 2 (degrees).
      *
      * The GeodesicLineExact::mask values possible for \e outmask are
      * - \e outmask |= GeodesicLineExact::LATITUDE for the latitude \e lat2;
@@ -467,20 +500,64 @@ namespace GeographicLib {
      * - \e outmask |= GeodesicLineExact::GEODESICSCALE for the geodesic scales
      *   \e M12 and \e M21;
      * - \e outmask |= GeodesicLineExact::AREA for the area \e S12;
-     * - \e outmask |= GeodesicLine::ALL for all of the above.
+     * - \e outmask |= GeodesicLineExact::ALL for all of the above;
+     * - \e outmask |= GeodesicLineExact::LONG_UNROLL to unroll \e lon2 instead
+     *   of wrapping it into the range [&minus;180&deg;, 180&deg;].
      * .
      * Requesting a value which the GeodesicLineExact object is not capable of
      * computing is not an error; the corresponding argument will not be
      * altered.  Note, however, that the arc length is always computed and
      * returned as the function value.
+     *
+     * With the GeodesicLineExact::LONG_UNROLL bit set, the quantity \e lon2
+     * &minus; \e lon1 indicates how many times and in what sense the geodesic
+     * encircles the ellipsoid.
      **********************************************************************/
-    Math::real GenPosition(bool arcmode, real s12_a12, unsigned outmask,
-                           real& lat2, real& lon2, real& azi2,
-                           real& s12, real& m12, real& M12, real& M21,
-                           real& S12) const throw();
-
+    Math::real GEOGRAPHICLIB_EXPORT
+    GenPosition(bool arcmode, real s12_a12, unsigned outmask,
+                real& lat2, real& lon2, real& azi2,
+                real& s12, real& m12, real& M12, real& M21, real& S12) const;
     ///@}
 
+    /** \name Setting point 3
+     **********************************************************************/
+    ///@{
+
+    /**
+     * Specify position of point 3 in terms of distance.
+     *
+     * @param[in] s13 the distance from point 1 to point 3 (meters); it
+     *   can be negative.
+     *
+     * This is only useful if the GeodesicLineExact object has been constructed
+     * with \e caps |= GeodesicLineExact::DISTANCE_IN.
+     **********************************************************************/
+    void GEOGRAPHICLIB_EXPORT SetDistance(real s13);
+
+    /**
+     * Specify position of point 3 in terms of arc length.
+     *
+     * @param[in] a13 the arc length from point 1 to point 3 (degrees); it
+     *   can be negative.
+     *
+     * The distance \e s13 is only set if the GeodesicLineExact object has been
+     * constructed with \e caps |= GeodesicLineExact::DISTANCE.
+     **********************************************************************/
+    void GEOGRAPHICLIB_EXPORT SetArc(real a13);
+
+    /**
+     * Specify position of point 3 in terms of either distance or arc length.
+     *
+     * @param[in] arcmode boolean flag determining the meaning of the second
+     *   parameter; if \e arcmode is false, then the GeodesicLineExact object
+     *   must have been constructed with \e caps |=
+     *   GeodesicLineExact::DISTANCE_IN.
+     * @param[in] s13_a13 if \e arcmode is false, this is the distance from
+     *   point 1 to point 3 (meters); otherwise it is the arc length from
+     *   point 1 to point 3 (degrees); it can be negative.
+     **********************************************************************/
+    void GEOGRAPHICLIB_EXPORT GenSetDistance(bool arcmode, real s13_a13);
+
     /** \name Inspector functions
      **********************************************************************/
     ///@{
@@ -488,42 +565,62 @@ namespace GeographicLib {
     /**
      * @return true if the object has been initialized.
      **********************************************************************/
-    bool Init() const throw() { return _caps != 0U; }
+    bool GEOGRAPHICLIB_EXPORT Init() const { return _caps != 0U; }
 
     /**
      * @return \e lat1 the latitude of point 1 (degrees).
      **********************************************************************/
-    Math::real Latitude() const throw()
-    { return Init() ? _lat1 : Math::NaN<real>(); }
+    Math::real GEOGRAPHICLIB_EXPORT Latitude() const
+    { return Init() ? _lat1 : Math::NaN(); }
 
     /**
      * @return \e lon1 the longitude of point 1 (degrees).
      **********************************************************************/
-    Math::real Longitude() const throw()
-    { return Init() ? _lon1 : Math::NaN<real>(); }
+    Math::real GEOGRAPHICLIB_EXPORT Longitude() const
+    { return Init() ? _lon1 : Math::NaN(); }
 
     /**
      * @return \e azi1 the azimuth (degrees) of the geodesic line at point 1.
      **********************************************************************/
-    Math::real Azimuth() const throw()
-    { return Init() ? _azi1 : Math::NaN<real>(); }
+    Math::real GEOGRAPHICLIB_EXPORT Azimuth() const
+    { return Init() ? _azi1 : Math::NaN(); }
+
+    /**
+     * The sine and cosine of \e azi1.
+     *
+     * @param[out] sazi1 the sine of \e azi1.
+     * @param[out] cazi1 the cosine of \e azi1.
+     **********************************************************************/
+    void GEOGRAPHICLIB_EXPORT Azimuth(real& sazi1, real& cazi1) const
+    { if (Init()) { sazi1 = _salp1; cazi1 = _calp1; } }
 
     /**
      * @return \e azi0 the azimuth (degrees) of the geodesic line as it crosses
-     * the equator in a northward direction.
+     *   the equator in a northward direction.
+     *
+     * The result lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real EquatorialAzimuth() const throw() {
-      return Init() ?
-        atan2(_salp0, _calp0) / Math::degree<real>() : Math::NaN<real>();
-    }
+    Math::real GEOGRAPHICLIB_EXPORT EquatorialAzimuth() const
+    { return Init() ? Math::atan2d(_salp0, _calp0) : Math::NaN(); }
+
+    /**
+     * The sine and cosine of \e azi0.
+     *
+     * @param[out] sazi0 the sine of \e azi0.
+     * @param[out] cazi0 the cosine of \e azi0.
+     **********************************************************************/
+    void GEOGRAPHICLIB_EXPORT EquatorialAzimuth(real& sazi0, real& cazi0) const
+    { if (Init()) { sazi0 = _salp0; cazi0 = _calp0; } }
 
     /**
      * @return \e a1 the arc length (degrees) between the northward equatorial
-     * crossing and point 1.
+     *   crossing and point 1.
+     *
+     * The result lies in [&minus;180&deg;, 180&deg;].
      **********************************************************************/
-    Math::real EquatorialArc() const throw() {
-      return Init() ?
-        atan2(_ssig1, _csig1) / Math::degree<real>() : Math::NaN<real>();
+    Math::real GEOGRAPHICLIB_EXPORT EquatorialArc() const {
+      using std::atan2;
+      return Init() ? atan2(_ssig1, _csig1) / Math::degree() : Math::NaN();
     }
 
     /**
@@ -531,39 +628,53 @@ namespace GeographicLib {
      *   the value inherited from the GeodesicExact object used in the
      *   constructor.
      **********************************************************************/
-    Math::real MajorRadius() const throw()
-    { return Init() ? _a : Math::NaN<real>(); }
+    Math::real GEOGRAPHICLIB_EXPORT EquatorialRadius() const
+    { return Init() ? _a : Math::NaN(); }
 
     /**
      * @return \e f the flattening of the ellipsoid.  This is the value
      *   inherited from the GeodesicExact object used in the constructor.
      **********************************************************************/
-    Math::real Flattening() const throw()
-    { return Init() ? _f : Math::NaN<real>(); }
-
-    /// \cond SKIP
-    /**
-     * <b>DEPRECATED</b>
-     * @return \e r the inverse flattening of the ellipsoid.
-     **********************************************************************/
-    Math::real InverseFlattening() const throw()
-    { return Init() ? 1/_f : Math::NaN<real>(); }
-    /// \endcond
+    Math::real GEOGRAPHICLIB_EXPORT Flattening() const
+    { return Init() ? _f : Math::NaN(); }
 
     /**
      * @return \e caps the computational capabilities that this object was
      *   constructed with.  LATITUDE and AZIMUTH are always included.
      **********************************************************************/
-    unsigned Capabilities() const throw() { return _caps; }
+    unsigned GEOGRAPHICLIB_EXPORT Capabilities() const { return _caps; }
 
     /**
+     * Test what capabilities are available.
+     *
      * @param[in] testcaps a set of bitor'ed GeodesicLineExact::mask values.
      * @return true if the GeodesicLineExact object has all these capabilities.
      **********************************************************************/
-    bool Capabilities(unsigned testcaps) const throw() {
+    bool GEOGRAPHICLIB_EXPORT Capabilities(unsigned testcaps) const {
       testcaps &= OUT_ALL;
       return (_caps & testcaps) == testcaps;
     }
+
+    /**
+     * The distance or arc length to point 3.
+     *
+     * @param[in] arcmode boolean flag determining the meaning of returned
+     *   value.
+     * @return \e s13 if \e arcmode is false; \e a13 if \e arcmode is true.
+     **********************************************************************/
+    Math::real GEOGRAPHICLIB_EXPORT GenDistance(bool arcmode) const
+    { return Init() ? (arcmode ? _a13 : _s13) : Math::NaN(); }
+
+    /**
+     * @return \e s13, the distance to point 3 (meters).
+     **********************************************************************/
+    Math::real GEOGRAPHICLIB_EXPORT Distance() const
+    { return GenDistance(false); }
+
+    /**
+     * @return \e a13, the arc length to point 3 (degrees).
+     **********************************************************************/
+    Math::real GEOGRAPHICLIB_EXPORT Arc() const { return GenDistance(true); }
     ///@}
 
   };
