@@ -2,25 +2,11 @@
  * \file Planimeter.cpp
  * \brief Command line utility for measuring the area of geodesic polygons
  *
- * Copyright (c) Charles Karney (2010-2012) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2010-2023) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
- * http://geographiclib.sourceforge.net/
+ * https://geographiclib.sourceforge.io/
  *
- * Compile and link with
- *   g++ -g -O3 -I../include -I../man -o Planimeter \
- *       Planimeter.cpp \
- *       ../src/DMS.cpp \
- *       ../src/GeoCoords.cpp \
- *       ../src/Geodesic.cpp \
- *       ../src/GeodesicLine.cpp \
- *       ../src/MGRS.cpp \
- *       ../src/PolarStereographic.cpp \
- *       ../src/PolygonArea.cpp \
- *       ../src/TransverseMercator.cpp \
- *       ../src/UTMUPS.cpp
- *
- * See the <a href="Planimeter.1.html">man page</a> for usage
- * information.
+ * See the <a href="Planimeter.1.html">man page</a> for usage information.
  **********************************************************************/
 
 #include <iostream>
@@ -31,17 +17,28 @@
 #include <GeographicLib/DMS.hpp>
 #include <GeographicLib/Utility.hpp>
 #include <GeographicLib/GeoCoords.hpp>
+#include <GeographicLib/AuxLatitude.hpp>
+
+#if defined(_MSC_VER)
+// Squelch warnings about constant conditional expressions
+#  pragma warning (disable: 4127)
+#endif
 
 #include "Planimeter.usage"
 
-int main(int argc, char* argv[]) {
+int main(int argc, const char* const argv[]) {
   try {
     using namespace GeographicLib;
     typedef Math::real real;
+    Utility::set_digits();
+    enum { GEODESIC, AUTHALIC, RHUMB, EXACT };
     real
-      a = Constants::WGS84_a<real>(),
-      f = Constants::WGS84_f<real>();
-    bool reverse = false, sign = true, polyline = false;
+      a = Constants::WGS84_a(),
+      f = Constants::WGS84_f();
+    bool reverse = false, sign = true, polyline = false, longfirst = false,
+      exact = false, geoconvert_compat = false;
+    int linetype = GEODESIC;
+    int prec = 6;
     std::string istring, ifile, ofile, cdelim;
     char lsep = ';';
 
@@ -56,7 +53,7 @@ int main(int argc, char* argv[]) {
       else if (arg == "-e") {
         if (m + 2 >= argc) return usage(1, true);
         try {
-          a = Utility::num<real>(std::string(argv[m + 1]));
+          a = Utility::val<real>(std::string(argv[m + 1]));
           f = Utility::fract<real>(std::string(argv[m + 2]));
         }
         catch (const std::exception& e) {
@@ -64,7 +61,28 @@ int main(int argc, char* argv[]) {
           return 1;
         }
         m += 2;
-      } else if (arg == "--input-string") {
+      } else if (arg == "-w")
+        longfirst = !longfirst;
+      else if (arg == "-p") {
+        if (++m == argc) return usage(1, true);
+        try {
+          prec = Utility::val<int>(std::string(argv[m]));
+        }
+        catch (const std::exception&) {
+          std::cerr << "Precision " << argv[m] << " is not a number\n";
+          return 1;
+        }
+      } else if (arg == "-G")
+        linetype = GEODESIC;
+      else if (arg == "-Q")
+        linetype = AUTHALIC;
+      else if (arg == "-R")
+        linetype = RHUMB;
+      else if (arg == "-E")
+        exact = true;
+      else if (arg == "--geoconvert-input")
+        geoconvert_compat = true;
+      else if (arg == "--input-string") {
         if (++m == argc) return usage(1, true);
         istring = argv[m];
       } else if (arg == "--input-file") {
@@ -84,10 +102,8 @@ int main(int argc, char* argv[]) {
         if (++m == argc) return usage(1, true);
         cdelim = argv[m];
       } else if (arg == "--version") {
-        std::cout
-          << argv[0]
-          << ": $Id: dbb8a9a0d79f172256044e72f65ebc667801eeac $\n"
-          << "GeographicLib version " << GEOGRAPHICLIB_VERSION_STRING << "\n";
+        std::cout << argv[0] << ": GeographicLib version "
+                  << GEOGRAPHICLIB_VERSION_STRING << "\n";
         return 0;
       } else
         return usage(!(arg == "-h" || arg == "--help"), arg != "--help");
@@ -130,14 +146,33 @@ int main(int argc, char* argv[]) {
     }
     std::ostream* output = !ofile.empty() ? &outfile : &std::cout;
 
+    // At start linetype is one of GEODESIC, AUTHALIC, RHUMB
+    const AuxLatitude ellip(a, f);
+    if (linetype == AUTHALIC) {
+      linetype = exact ? EXACT : GEODESIC;
+      using std::sqrt;
+      a = sqrt(ellip.AuthalicRadiusSquared(exact));
+      f = 0;
+    } else if (linetype == GEODESIC && exact)
+      linetype = EXACT;
+    // Now linetype is one of GEODESIC, EXACT, RHUMB
     const Geodesic geod(a, f);
+    const GeodesicExact geode(a, linetype == EXACT ? f : 0);
+    const Rhumb rhumb(a, f, exact && linetype == RHUMB);
     PolygonArea poly(geod, polyline);
+    PolygonAreaExact polye(geode, polyline);
+    PolygonAreaRhumb polyr(rhumb, polyline);
     GeoCoords p;
 
-    std::string s;
+    // Max precision = 10: 0.1 nm in distance, 10^-15 deg (= 0.11 nm),
+    // 10^-11 sec (= 0.3 nm).
+    prec = std::min(10 + Math::extra_digits(), std::max(0, prec));
+    std::string s, eol("\n");
     real perimeter, area;
     unsigned num;
-    std::string eol("\n");
+    std::istringstream str;
+    std::string slat, slon, junk;
+    real lat = 0, lon = 0;
     while (std::getline(*input, s)) {
       if (!cdelim.empty()) {
         std::string::size_type m = s.find(cdelim);
@@ -149,8 +184,19 @@ int main(int argc, char* argv[]) {
       bool endpoly = s.empty();
       if (!endpoly) {
         try {
-          p.Reset(s);
-          if (Math::isnan(p.Latitude()) || Math::isnan(p.Longitude()))
+          using std::isnan;
+          if (geoconvert_compat) {
+            p.Reset(s, true, longfirst);
+            lat = p.Latitude(); lon = p.Longitude();
+          } else {
+            str.clear(); str.str(s);
+            if (!(str >> slat >> slon))
+              throw GeographicErr("incomplete input");
+            if (str >> junk)
+              throw GeographicErr("extra input");
+            DMS::DecodeLatLon(slat, slon, lat, lon, longfirst);
+          }
+          if (isnan(lat) || isnan(lon))
             endpoly = true;
         }
         catch (const GeographicErr&) {
@@ -158,28 +204,42 @@ int main(int argc, char* argv[]) {
         }
       }
       if (endpoly) {
-        num = poly.Compute(reverse, sign, perimeter, area);
+        num =
+          linetype == EXACT ? polye.Compute(reverse, sign, perimeter, area) :
+          linetype == RHUMB ? polyr.Compute(reverse, sign, perimeter, area) :
+          poly.Compute(reverse, sign, perimeter, area); // geodesic + authalic
         if (num > 0) {
-          *output << num << " "
-                  << Utility::str<real>(perimeter, 8);
-          if (!polyline)
-            *output << " " << Utility::str<real>(area, 3);
+          *output << num << " " << Utility::str(perimeter, prec);
+          if (!polyline) {
+            *output << " " << Utility::str(area, std::max(0, prec - 5));
+          }
           *output << eol;
         }
-        poly.Clear();
+        linetype == EXACT ? polye.Clear() :
+          linetype == RHUMB ? polyr.Clear() : poly.Clear();
         eol = "\n";
-      } else
-        poly.AddPoint(p.Latitude(), p.Longitude());
+      } else {
+        linetype == EXACT ? polye.AddPoint(lat, lon) :
+          linetype == RHUMB ? polyr.AddPoint(lat, lon) :
+          poly.AddPoint
+          (linetype == AUTHALIC ?
+           ellip.Convert(AuxLatitude::PHI, AuxLatitude::XI, lat, exact) : lat,
+           lon);
+      }
     }
-    num = poly.Compute(reverse, sign, perimeter, area);
+    num =
+      linetype == EXACT ? polye.Compute(reverse, sign, perimeter, area) :
+      linetype == RHUMB ? polyr.Compute(reverse, sign, perimeter, area) :
+      poly.Compute(reverse, sign, perimeter, area);
     if (num > 0) {
-      *output << num << " "
-              << Utility::str<real>(perimeter, 8);
-      if (!polyline)
-        *output << " " << Utility::str<real>(area, 3);
+      *output << num << " " << Utility::str(perimeter, prec);
+      if (!polyline) {
+        *output << " " << Utility::str(area, std::max(0, prec - 5));
+      }
       *output << eol;
     }
-    poly.Clear();
+    linetype == EXACT ? polye.Clear() :
+      linetype == RHUMB ? polyr.Clear() : poly.Clear();
     eol = "\n";
     return 0;
   }
